@@ -39,12 +39,13 @@
 #include <limits.h>
 #include <libgen.h>
 #include <utime.h>
+#include <time.h>
 
 #include "nvidia-installer.h"
 #include "user-interface.h"
 #include "files.h"
 #include "misc.h"
-
+#include "precompiled.h"
 
 /*
  * remove_directory() - recursively delete a direcotry (`rm -rf`)
@@ -299,6 +300,11 @@ int set_destinations(Options *op, Package *p)
             path = INSTALLER_BINARY_DST_PATH;
             break;
             
+        case FILE_TYPE_NVIDIA_SETTINGS_BINARY:
+            prefix = op->nvidia_settings_prefix;
+            path = NVIDIA_SETTINGS_BINARY_DST_PATH;
+            break;
+
         case FILE_TYPE_KERNEL_MODULE:
             
             /*
@@ -861,3 +867,143 @@ int check_for_existing_rpms(Options *op)
     return TRUE;
     
 } /* check_for_existing_rpms() */
+
+
+
+/*
+ * copy_directory_contents() - copy the contents of directory src to
+ * directory dst.  This only copies files; subdirectories are ignored.
+ */
+
+int copy_directory_contents(Options *op, const char *src, const char *dst)
+{
+    DIR *dir;
+    struct dirent *ent;
+    char *srcfile, *dstfile;
+    struct stat stat_buf;
+
+    if ((dir = opendir(src)) == NULL) {
+        ui_error(op, "Unable to open directory '%s' (%s).",
+                 src, strerror(errno));
+        return FALSE;
+    }
+
+    while ((ent = readdir(dir)) != NULL) {
+        
+        if (((strcmp(ent->d_name, ".")) == 0) ||
+            ((strcmp(ent->d_name, "..")) == 0)) continue;
+        
+        srcfile = nvstrcat(src, "/", ent->d_name, NULL);
+
+        /* only copy regular files */
+
+        if ((stat(srcfile, &stat_buf) == -1) || !(S_ISREG(stat_buf.st_mode))) {
+            nvfree(srcfile);
+            continue;
+        }
+        
+        dstfile = nvstrcat(dst, "/", ent->d_name, NULL);
+        
+        if (!copy_file(op, srcfile, dstfile, stat_buf.st_mode)) return FALSE;
+
+        nvfree(srcfile);
+        nvfree(dstfile);
+    }
+
+    if (closedir(dir) != 0) {
+        ui_error(op, "Failure while closing directory '%s' (%s).",
+                 src, strerror(errno));
+        
+        return FALSE;
+    }
+    
+    return TRUE;
+    
+} /* copy_directory_contents() */
+
+
+
+/*
+ * pack_precompiled_kernel_interface() - 
+ */
+
+int pack_precompiled_kernel_interface(Options *op, Package *p)
+{
+    char *cmd, time_str[256], *proc_version_string;
+    char major[16], minor[16], patch[16];
+    char *result, *descr;
+    time_t t;
+    struct utsname buf;
+    int ret;
+
+    ui_log(op, "Packaging precompiled kernel interface.");
+
+    /* make sure the precompiled_kernel_interface_directory exists */
+
+    mkdir_recursive(op, p->precompiled_kernel_interface_directory, 0755);
+    
+    /* use the time in the output string... should be fairly unique */
+
+    t = time(NULL);
+    snprintf(time_str, 256, "%lu", t);
+    
+    /* read the proc version string */
+
+    proc_version_string = read_proc_version(op);
+
+    /* get the version strings */
+
+    snprintf(major, 16, "%d", p->major);
+    snprintf(minor, 16, "%d", p->minor);
+    snprintf(patch, 16, "%d", p->patch);
+    
+    /* use the uname string as the description */
+
+    uname(&buf);
+    descr = nvstrcat(buf.sysname, " ",
+                     buf.release, " ",
+                     buf.version, " ",
+                     buf.machine, NULL);
+    
+    /* build the mkprecompiled command */
+
+    cmd = nvstrcat("./usr/bin/mkprecompiled --interface=",
+                   p->kernel_module_build_directory, "/",
+                   PRECOMPILED_KERNEL_INTERFACE_FILENAME,
+                   " --output=", p->precompiled_kernel_interface_directory,
+                   "/", PRECOMPILED_KERNEL_INTERFACE_FILENAME,
+                   "-", p->version_string, ".", time_str,
+                   " --description=\"", descr, "\"",
+                   " --proc-version=\"", proc_version_string, "\"",
+                   " --major=", major,
+                   " --minor=", minor,
+                   " --patch=", patch, NULL);
+
+    /* execute the command */
+    
+    ret = run_command(op, cmd, &result, FALSE, 0);
+    
+    nvfree(cmd);
+    nvfree(proc_version_string);
+    nvfree(descr);
+    
+    /* remove the old kernel interface file */
+
+    cmd = nvstrcat(p->kernel_module_build_directory, "/",
+                   PRECOMPILED_KERNEL_INTERFACE_FILENAME, NULL);
+    
+    unlink(cmd); /* XXX what to do if this fails? */
+
+    nvfree(cmd);
+    
+    if (ret != 0) {
+        ui_error(op, "Unable to package precompiled kernel interface: %s",
+                 result);
+    }
+
+    nvfree(result);
+
+    if (ret == 0) return TRUE;
+    else return FALSE;
+    
+} /* pack_kernel_interface() */
