@@ -270,15 +270,42 @@ int install_from_cwd(Options *op)
 
 
 
+/*
+ * add_this_kernel() - build a precompiled kernel interface for the
+ * running kernel, and repackage the .run file to include the new
+ * precompiled kernel interface.
+ */
 
+int add_this_kernel(Options *op)
+{
+    Package *p;
+    
+    /* parse the manifest */
 
+    if ((p = parse_manifest(op)) == NULL) goto failed;
 
+    /* find the kernel header files */
 
+    if (!determine_kernel_include_path(op)) goto failed;
 
+    /* build the precompiled kernel interface */
 
+    if (!build_kernel_interface(op, p)) goto failed;
+    
+    /* pack the precompiled kernel interface */
 
+    if (!pack_precompiled_kernel_interface(op, p)) goto failed;
+    
+    return TRUE;
 
+ failed:
 
+    ui_error(op, "Unable to add a precompiled kernel interface for the "
+             "running kernel.");
+    
+    return FALSE;
+    
+} /* add_this_kernel() */
 
 
 
@@ -286,11 +313,12 @@ int install_from_cwd(Options *op)
  * parse_manifest() - open and read the .manifest file in the current
  * directory.
  *
- * The first seven lines of the .manifest file are:
+ * The first nine lines of the .manifest file are:
  *
  *   - a description string
  *   - a version string of the form "major.minor-patch"
  *   - the kernel module file name
+ *   - the kernel interface file name
  *   - the kernel module name (what `rmmod` and `modprobe` should use)
  *   - a whitespace-separated list of module names that should be
  *     removed before installing a new kernel module
@@ -309,7 +337,7 @@ int install_from_cwd(Options *op)
  *   - symbolic links will name the target of the link
  */
 
-Package *parse_manifest (Options *op)
+static Package *parse_manifest (Options *op)
 {
     char *buf, *c, *flag , *tmpstr;
     int done, n, line;
@@ -356,14 +384,20 @@ Package *parse_manifest (Options *op)
     p->kernel_module_filename = get_next_line(ptr, &ptr);
     if (!p->kernel_module_filename) goto invalid_manifest_file;
 
-    /* the fourth line is the kernel module name */
+    /* new fourth line is the kernel interface filename */
+
+    line++;
+    p->kernel_interface_filename = get_next_line(ptr, &ptr);
+    if (!p->kernel_module_filename) goto invalid_manifest_file;
+
+    /* the fifth line is the kernel module name */
 
     line++;
     p->kernel_module_name = get_next_line(ptr, &ptr);
     if (!p->kernel_module_name) goto invalid_manifest_file;
 
     /*
-     * the fifth line is a whitespace-separated list of kernel modules
+     * the sixth line is a whitespace-separated list of kernel modules
      * to be unloaded before installing the new kernel module
      */
 
@@ -383,7 +417,7 @@ Package *parse_manifest (Options *op)
     } while (p->bad_modules[n-1]);
     
     /*
-     * the sixth line is a whitespace-separated list of kernel module
+     * the seventh line is a whitespace-separated list of kernel module
      * filenames to be uninstalled before installing the new kernel
      * module
      */
@@ -403,7 +437,7 @@ Package *parse_manifest (Options *op)
         p->bad_module_filenames[n-1] = read_next_word(c, &c);
     } while (p->bad_module_filenames[n-1]);
     
-    /* the seventh line is the kernel module build directory */
+    /* the eighth line is the kernel module build directory */
 
     line++;
     p->kernel_module_build_directory = get_next_line(ptr, &ptr);
@@ -411,7 +445,7 @@ Package *parse_manifest (Options *op)
     remove_trailing_slashes(p->kernel_module_build_directory);
 
     /*
-     * the eighth line is the directory containing precompiled kernel
+     * the nineth line is the directory containing precompiled kernel
      * interfaces
      */
 
@@ -487,6 +521,8 @@ Package *parse_manifest (Options *op)
                 p->entries[n].flags |= FILE_TYPE_XFREE86_SYMLINK;
             else if (strcmp(flag, "INSTALLER_BINARY") == 0)
                 p->entries[n].flags |= FILE_TYPE_INSTALLER_BINARY;
+            else if (strcmp(flag, "NVIDIA_SETTINGS_BINARY") == 0)
+                p->entries[n].flags |= FILE_TYPE_NVIDIA_SETTINGS_BINARY;
             else if (strcmp(flag, "OPENGL_LIB_CLASSIC_TLS") == 0)
                 p->entries[n].flags |= (FILE_TYPE_OPENGL_LIB |
                                         FILE_CLASS_CLASSIC_TLS);
