@@ -352,6 +352,107 @@ int build_kernel_module(Options *op, Package *p)
 } /* build_kernel_module() */
 
 
+
+/*
+ * build_kernel_interface() - build the kernel interface, and place it
+ * here:
+ *
+ * "%s/%s", p->kernel_module_build_directory,
+ * PRECOMPILED_KERNEL_INTERFACE_FILENAME
+ *
+ * This is done by copying the sources to a temporary working
+ * directory, building, and copying the kernel interface back to the
+ * kernel module source directory.  The tmpdir is removed when
+ * complete.
+ *
+ * XXX this and build_kernel_module() should be merged.
+ */
+
+int build_kernel_interface(Options *op, Package *p)
+{
+    char *tmpdir = NULL;
+    char *cmd = NULL;
+    char *kernel_interface = NULL;
+    char *dstfile = NULL;
+    int ret = FALSE;
+    int command_ret;
+
+    /* create a temporary directory */
+
+    tmpdir = nvstrcat(op->tmpdir, "/nv-src-XXXXXX", NULL);
+
+    if (mkdtemp(tmpdir) == NULL) {
+        ui_error(op, "Unable to create a temporary build directory (%s).",
+                 strerror(errno));
+        return FALSE;
+    }
+    
+    /* copy the kernel module sources to it */
+    
+    ui_log(op, "Copying kernel module sources to temporary directory.");
+
+    if (!copy_directory_contents
+        (op, p->kernel_module_build_directory, tmpdir)) {
+        ui_error(op, "Unable to copy the kernel module sources to temporary "
+                 "directory '%s'.", tmpdir);
+        goto failed;
+    }
+    
+    /* build the kernel interface */
+
+    ui_status_begin(op, "Building kernel interface:", "Building");
+    
+    cmd = nvstrcat("cd ", tmpdir, "; make ", p->kernel_interface_filename,
+                   " SYSINCLUDE=", op->kernel_include_path, NULL);
+    
+    command_ret = run_command(op, cmd, NULL, TRUE, 25 /* XXX */);
+
+    if (command_ret != 0) {
+        ui_status_end(op, "Error.");
+        ui_error(op, "Unable to build the NVIDIA kernel module interface.");
+        /* XXX need more descriptive error message */
+        goto failed;
+    }
+    
+    /* check that the file exists */
+
+    kernel_interface = nvstrcat(tmpdir, "/",
+                                p->kernel_interface_filename, NULL);
+    
+    if (access(kernel_interface, F_OK) == -1) {
+        ui_status_end(op, "Error.");
+        ui_error(op, "The NVIDIA kernel module interface was not created.");
+        goto failed;
+    }
+
+    ui_status_end(op, "done.");
+
+    ui_log(op, "Kernel module interface compilation complete.");
+
+    /* copy the kernel interface from the tmpdir back to the srcdir */
+    
+    dstfile = nvstrcat(p->kernel_module_build_directory, "/",
+                       PRECOMPILED_KERNEL_INTERFACE_FILENAME, NULL);
+
+    if (!copy_file(op, kernel_interface, dstfile, 0644)) goto failed;
+    
+    ret = TRUE;
+    
+ failed:
+    
+    remove_directory(op, tmpdir);
+
+    if (tmpdir) nvfree(tmpdir);
+    if (cmd) nvfree(cmd);
+    if (kernel_interface) nvfree(kernel_interface);
+    if (dstfile) nvfree(dstfile);
+
+    return ret;
+
+} /* build_kernel_interface() */
+
+
+
 /*
  * test_kernel_module() - attempt to insmod the kernel module and then
  * rmmod it.  Return TRUE if the insmod succeeded, or FALSE otherwise.
@@ -679,6 +780,7 @@ int find_precompiled_kernel_interface(Options *op, Package *p)
 
 
 
+
 /*
  ***************************************************************************
  * local static routines
