@@ -238,6 +238,12 @@ static void print_advanced_options_args_only(int args_only)
             DEFAULT_INSTALLER_INSTALLATION_PREFIX);
     fmtout("");
 
+    fmtout("--nvidia-settings-prefix=[NVIDIA-SETTINGS PREFIX]");
+    fmtoutp(TAB, "The prefix under which the NVIDIA XFree86 Settings Utility "
+            "(nvidia-settings) will be installed; the default is: '%s'.",
+            DEFAULT_NVIDIA_SETTINGS_INSTALLATION_PREFIX);
+    fmtout("");
+
     fmtout("--kernel-include-path=[KERNEL INCLUDE PATH]");
     fmtoutp(TAB, "The directory containing the kernel header files that "
             "should be used when compiling the NVIDIA kernel module.  "
@@ -332,7 +338,7 @@ static void print_advanced_options_args_only(int args_only)
             "actually running).  This option implies "
             "'--no-precompiled-interface'.  If the options "
             "'--kernel-install-path' and '--kernel-include-path' are not "
-            "given, then they will be infered from [KERNELNAME]; eg: "
+            "given, then they will be inferred from [KERNELNAME]; eg: "
             "'/lib/modules/[KERNELNAME]/kernel/drivers/video/' and "
             "'/lib/modules/[KERNELNAME]/build/include/', respectively.");
     fmtout("");
@@ -378,6 +384,8 @@ Options *parse_commandline(int argc, char *argv[])
 #define FORCE_TLS_OPTION                13
 #define SANITY_OPTION                   14                 
 #define ADVANCED_OPTIONS_ARGS_ONLY_OPTION 15
+#define NVIDIA_SETTINGS_PREFIX_OPTION   16
+#define ADD_THIS_KERNEL_OPTION          17
 
 
     static struct option long_options[] = {
@@ -400,6 +408,7 @@ Options *parse_commandline(int argc, char *argv[])
         { "xfree86-prefix",           1, NULL, XFREE86_PREFIX_OPTION      },
         { "opengl-prefix",            1, NULL, OPENGL_PREFIX_OPTION       },
         { "installer-prefix",         1, NULL, INSTALLER_PREFIX_OPTION    },
+        { "nvidia-settings-prefix",   1, NULL, NVIDIA_SETTINGS_PREFIX_OPTION},
         { "kernel-include-path",      1, NULL, KERNEL_INCLUDE_PATH_OPTION },
         { "kernel-install-path",      1, NULL, KERNEL_INSTALL_PATH_OPTION },
         { "uninstall",                0, NULL, UNINSTALL_OPTION           },
@@ -411,7 +420,9 @@ Options *parse_commandline(int argc, char *argv[])
         { "opengl-headers",           0, NULL, OPENGL_HEADERS_OPTION      },
         { "force-tls",                1, NULL, FORCE_TLS_OPTION           },
         { "sanity",                   0, NULL, SANITY_OPTION              },
-        { "advanced-options-args-only", 0, NULL, ADVANCED_OPTIONS_ARGS_ONLY_OPTION},
+        { "add-this-kernel",          0, NULL, ADD_THIS_KERNEL_OPTION     },
+        { "advanced-options-args-only", 0, NULL,
+          ADVANCED_OPTIONS_ARGS_ONLY_OPTION},
         {  0,                         0, NULL, 0                          }
     };
     
@@ -422,6 +433,7 @@ Options *parse_commandline(int argc, char *argv[])
     op->xfree86_prefix = DEFAULT_XFREE86_INSTALLATION_PREFIX;
     op->opengl_prefix = DEFAULT_OPENGL_INSTALLATION_PREFIX;
     op->installer_prefix = DEFAULT_INSTALLER_INSTALLATION_PREFIX;
+    op->nvidia_settings_prefix = DEFAULT_NVIDIA_SETTINGS_INSTALLATION_PREFIX;
     op->proc_mount_point = DEFAULT_PROC_MOUNT_POINT;
     op->log_file_name = DEFAULT_LOG_FILE_NAME;
     op->ftp_site = DEFAULT_FTP_SITE;
@@ -472,6 +484,8 @@ Options *parse_commandline(int argc, char *argv[])
             op->opengl_prefix = optarg; break;
         case INSTALLER_PREFIX_OPTION:
             op->installer_prefix = optarg; break;
+        case NVIDIA_SETTINGS_PREFIX_OPTION:
+            op->nvidia_settings_prefix = optarg; break;
         case KERNEL_INCLUDE_PATH_OPTION:
             op->kernel_include_path = optarg; break;
         case KERNEL_INSTALL_PATH_OPTION:
@@ -506,6 +520,9 @@ Options *parse_commandline(int argc, char *argv[])
         case SANITY_OPTION:
             op->sanity = TRUE;
             break;
+        case ADD_THIS_KERNEL_OPTION:
+            op->add_this_kernel = TRUE;
+            break;
         case ADVANCED_OPTIONS_ARGS_ONLY_OPTION:
             print_advanced_options_args_only(TRUE); exit(0); break;
             
@@ -609,6 +626,12 @@ int main(int argc, char *argv[])
         ret = update(op);
     }
 
+    /* add this kernel */
+
+    else if (op->add_this_kernel) {
+        ret = add_this_kernel(op);
+    }
+
     /* install from the cwd */
     
     else {
